@@ -10,29 +10,34 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+import json
+import os
+import platform
 import sys
-import signal
 import logging
 
+import distro
 import botocore.session
-from botocore import __version__ as botocore_version
-from botocore.hooks import HierarchicalEmitter
 from botocore import xform_name
 from botocore.compat import copy_kwargs, OrderedDict
-from botocore.exceptions import NoCredentialsError
-from botocore.exceptions import NoRegionError
 from botocore.history import get_global_history_recorder
+from botocore.configprovider import InstanceVarProvider
+from botocore.configprovider import EnvironmentProvider
+from botocore.configprovider import ScopedConfigProvider
+from botocore.configprovider import ConstantProvider
+from botocore.configprovider import ChainProvider
 
-from awscli import EnvironmentVariables, __version__
-from awscli.compat import get_stderr_text_writer
+from awscli import __version__
+from awscli.compat import (
+    default_pager, get_stderr_text_writer, get_stdout_text_writer
+)
 from awscli.formatter import get_formatter
 from awscli.plugin import load_plugins
 from awscli.commands import CLICommand
-from awscli.compat import six
 from awscli.argparser import MainArgParser
+from awscli.argparser import FirstPassGlobalArgParser
 from awscli.argparser import ServiceArgParser
 from awscli.argparser import ArgTableArgParser
-from awscli.argparser import USAGE
 from awscli.help import ProviderHelpCommand
 from awscli.help import ServiceHelpCommand
 from awscli.help import OperationHelpCommand
@@ -44,51 +49,296 @@ from awscli.arguments import UnknownArgumentError
 from awscli.argprocess import unpack_argument
 from awscli.alias import AliasLoader
 from awscli.alias import AliasCommandInjector
+from awscli.logger import (
+    set_stream_logger, remove_stream_logger, enable_crt_logging,
+    disable_crt_logging,
+)
 from awscli.utils import emit_top_level_args_parsed_event
-from awscli.utils import write_exception
+from awscli.utils import OutputStreamFactory
+from awscli.utils import IMDSRegionProvider
+from awscli.constants import PARAM_VALIDATION_ERROR_RC
+from awscli.autoprompt.core import AutoPromptDriver
+from awscli.errorhandler import (
+    construct_cli_error_handlers_chain, construct_entry_point_handlers_chain
+)
 
 
 LOG = logging.getLogger('awscli.clidriver')
 LOG_FORMAT = (
     '%(asctime)s - %(threadName)s - %(name)s - %(levelname)s - %(message)s')
 HISTORY_RECORDER = get_global_history_recorder()
+METADATA_FILENAME = 'metadata.json'
+# Don't remove this line.  The idna encoding
+# is used by getaddrinfo when dealing with unicode hostnames,
+# and in some cases, there appears to be a race condition
+# where threads will get a LookupError on getaddrinfo() saying
+# that the encoding doesn't exist.  Using the idna encoding before
+# running any CLI code (and any threads it may create) ensures that
+# the encodings.idna is imported and registered in the codecs registry,
+# which will stop the LookupErrors from happening.
+# See: https://bugs.python.org/issue29288
+u''.encode('idna')
 
 
 def main():
-    driver = create_clidriver()
-    rc = driver.main()
-    HISTORY_RECORDER.record('CLI_RC', rc, 'CLI')
-    return rc
+    return AWSCLIEntryPoint().main(sys.argv[1:])
 
 
-def create_clidriver():
-    session = botocore.session.Session(EnvironmentVariables)
+def create_clidriver(args=None):
+    debug = None
+    if args is not None:
+        parser = FirstPassGlobalArgParser()
+        args, _ = parser.parse_known_args(args)
+        debug = args.debug
+    session = botocore.session.Session()
     _set_user_agent_for_session(session)
     load_plugins(session.full_config.get('plugins', {}),
                  event_hooks=session.get_component('event_emitter'))
-    driver = CLIDriver(session=session)
+    error_handlers_chain = construct_cli_error_handlers_chain()
+    driver = CLIDriver(session=session,
+                       error_handler=error_handlers_chain,
+                       debug=debug)
     return driver
 
 
+def _get_distribution_source():
+    metadata_file = os.path.join(
+        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data'),
+        METADATA_FILENAME
+    )
+    metadata = {}
+    if os.path.isfile(metadata_file):
+        with open(metadata_file) as f:
+            metadata = json.load(f)
+    return metadata.get('distribution_source', 'other')
+
+
+def _get_distribution():
+    distribution = None
+    if platform.system() == 'Linux':
+        distribution = _get_linux_distribution()
+    return distribution
+
+
+def _get_linux_distribution():
+    linux_distribution = 'unknown'
+    try:
+        linux_distribution = distro.id()
+        version = distro.major_version()
+        if version:
+            linux_distribution += '.%s' % version
+    except Exception:
+        pass
+    return linux_distribution
+
+
 def _set_user_agent_for_session(session):
     session.user_agent_name = 'aws-cli'
     session.user_agent_version = __version__
-    session.user_agent_extra = 'botocore/%s' % botocore_version
+    # user_agent_extra on linux will look like "rpm/x86_64.Ubuntu.18"
+    # on mac and windows like "sources/x86_64"
+    session.user_agent_extra = '%s/%s' % (
+        _get_distribution_source(),
+        platform.machine()
+    )
+    linux_distribution = _get_distribution()
+    if linux_distribution:
+        session.user_agent_extra += ".%s" % linux_distribution
+
+
+def no_pager_handler(session, parsed_args, **kwargs):
+    if parsed_args.no_cli_pager:
+        config_store = session.get_component('config_store')
+        config_store.set_config_provider(
+            'pager', ConstantProvider(value=None)
+        )
+
+
+class AWSCLIEntryPoint:
+    def __init__(self, driver=None):
+        self._error_handler = construct_entry_point_handlers_chain()
+        self._driver = driver
+
+    def main(self, args):
+        try:
+            rc = self._do_main(args)
+        except BaseException as e:
+            LOG.debug("Exception caught in AWSCLIEntryPoint", exc_info=True)
+            return self._error_handler.handle_exception(
+                e,
+                stdout=get_stdout_text_writer(),
+                stderr=get_stderr_text_writer()
+            )
+
+        HISTORY_RECORDER.record('CLI_RC', rc, 'CLI')
+        return rc
+
+    def _run_driver(self, driver, args, prompt_mode):
+        driver.session.user_agent_extra += " prompt/%s" % prompt_mode
+        return driver.main(args)
+
+    def _do_main(self, args):
+        driver = self._driver
+        if driver is None:
+            driver = create_clidriver(args)
+        autoprompt_driver = AutoPromptDriver(driver)
+        auto_prompt_mode = autoprompt_driver.resolve_mode(args)
+        if auto_prompt_mode == 'on':
+            args = autoprompt_driver.prompt_for_args(args)
+            rc = self._run_driver(driver, args, prompt_mode='on')
+        elif auto_prompt_mode == 'on-partial':
+            autoprompt_driver.inject_silence_param_error_msg_handler(driver)
+            rc = self._run_driver(driver, args, prompt_mode='off')
+            if rc == PARAM_VALIDATION_ERROR_RC:
+                args = autoprompt_driver.prompt_for_args(args)
+                driver = create_clidriver(args)
+                rc = self._run_driver(driver, args, prompt_mode='partial')
+        else:
+            rc = self._run_driver(driver, args, prompt_mode='off')
+        return rc
 
 
 class CLIDriver(object):
 
-    def __init__(self, session=None):
+    def __init__(self, session=None, error_handler=None,
+                 debug=False):
         if session is None:
-            self.session = botocore.session.get_session(EnvironmentVariables)
+            self.session = botocore.session.get_session()
             _set_user_agent_for_session(self.session)
         else:
             self.session = session
+        self._error_handler = error_handler
+        if self._error_handler is None:
+            self._error_handler = construct_cli_error_handlers_chain()
+        if debug:
+            self._set_logging(debug)
+        self._update_config_chain()
         self._cli_data = None
         self._command_table = None
         self._argument_table = None
         self.alias_loader = AliasLoader()
 
+    def _update_config_chain(self):
+        config_store = self.session.get_component('config_store')
+        config_store.set_config_provider(
+            'region',
+            self._construct_cli_region_chain()
+        )
+        config_store.set_config_provider(
+            'output',
+            self._construct_cli_output_chain()
+        )
+        config_store.set_config_provider(
+            'pager',
+            self._construct_cli_pager_chain()
+        )
+        config_store.set_config_provider(
+            'cli_binary_format',
+            self._construct_cli_binary_format_chain()
+        )
+        config_store.set_config_provider(
+            'cli_auto_prompt',
+            self._construct_cli_auto_prompt_chain()
+        )
+
+    def _construct_cli_region_chain(self):
+        providers = [
+            InstanceVarProvider(
+                instance_var='region',
+                session=self.session
+            ),
+            EnvironmentProvider(
+                name='AWS_REGION',
+                env=os.environ,
+            ),
+            EnvironmentProvider(
+                name='AWS_DEFAULT_REGION',
+                env=os.environ,
+            ),
+            ScopedConfigProvider(
+                config_var_name='region',
+                session=self.session,
+            ),
+            IMDSRegionProvider(self.session),
+        ]
+        return ChainProvider(providers=providers)
+
+    def _construct_cli_output_chain(self):
+        providers = [
+            InstanceVarProvider(
+                instance_var='output',
+                session=self.session,
+            ),
+            EnvironmentProvider(
+                name='AWS_DEFAULT_OUTPUT',
+                env=os.environ,
+            ),
+            ScopedConfigProvider(
+                config_var_name='output',
+                session=self.session,
+            ),
+            ConstantProvider(value='json'),
+        ]
+        return ChainProvider(providers=providers)
+
+    def _construct_cli_pager_chain(self):
+        providers = [
+            EnvironmentProvider(
+                name='AWS_PAGER',
+                env=os.environ,
+            ),
+            ScopedConfigProvider(
+                config_var_name='cli_pager',
+                session=self.session,
+            ),
+            EnvironmentProvider(
+                name='PAGER',
+                env=os.environ,
+            ),
+            ConstantProvider(value=default_pager),
+        ]
+        return ChainProvider(providers=providers)
+
+    def _construct_cli_binary_format_chain(self):
+        providers = [
+            ScopedConfigProvider(
+                config_var_name='cli_binary_format',
+                session=self.session,
+            ),
+            ConstantProvider(value='base64'),
+        ]
+        return ChainProvider(providers=providers)
+
+    def _construct_cli_auto_prompt_chain(self):
+        providers = [
+            InstanceVarProvider(
+                instance_var='cli_auto_prompt',
+                session=self.session,
+            ),
+            EnvironmentProvider(
+                name='AWS_CLI_AUTO_PROMPT',
+                env=os.environ,
+            ),
+            ScopedConfigProvider(
+                config_var_name='cli_auto_prompt',
+                session=self.session
+            ),
+            ConstantProvider(value='off'),
+        ]
+        return ChainProvider(providers=providers)
+
+    @property
+    def subcommand_table(self):
+        return self._get_command_table()
+
+    @property
+    def arg_table(self):
+        return self._get_argument_table()
+
+    @property
+    def error_handler(self):
+        return self._error_handler
+
     def _get_cli_data(self):
         # Not crazy about this but the data in here is needed in
         # several places (e.g. MainArgParser, ProviderHelp) so
@@ -132,7 +382,6 @@ class CLIDriver(object):
         return commands
 
     def _add_aliases(self, command_table, parser):
-        parser = self._create_parser(command_table)
         injector = AliasCommandInjector(
             self.session, self.alias_loader)
         injector.inject_aliases(command_table, parser)
@@ -148,7 +397,9 @@ class CLIDriver(object):
         # Then the final step is to send out an event so handlers
         # can add extra arguments or modify existing arguments.
         self.session.emit('building-top-level-params',
-                          argument_table=argument_table)
+                          session=self.session,
+                          argument_table=argument_table,
+                          driver=self)
         return argument_table
 
     def _create_cli_argument(self, option_name, option_params):
@@ -169,7 +420,7 @@ class CLIDriver(object):
                                    cli_data.get('synopsis', None),
                                    cli_data.get('help_usage', None))
 
-    def _create_parser(self, command_table):
+    def create_parser(self, command_table):
         # Also add a 'help' command.
         command_table['help'] = self.create_help_command()
         cli_data = self._get_cli_data()
@@ -191,46 +442,31 @@ class CLIDriver(object):
         if args is None:
             args = sys.argv[1:]
         command_table = self._get_command_table()
-        parser = self._create_parser(command_table)
+        parser = self.create_parser(command_table)
         self._add_aliases(command_table, parser)
-        parsed_args, remaining = parser.parse_known_args(args)
         try:
             # Because _handle_top_level_args emits events, it's possible
             # that exceptions can be raised, which should have the same
             # general exception handling logic as calling into the
             # command table.  This is why it's in the try/except clause.
+            parsed_args, remaining = parser.parse_known_args(args)
             self._handle_top_level_args(parsed_args)
             self._emit_session_event(parsed_args)
             HISTORY_RECORDER.record(
                 'CLI_VERSION', self.session.user_agent(), 'CLI')
             HISTORY_RECORDER.record('CLI_ARGUMENTS', args, 'CLI')
             return command_table[parsed_args.command](remaining, parsed_args)
-        except UnknownArgumentError as e:
-            sys.stderr.write("usage: %s\n" % USAGE)
-            sys.stderr.write(str(e))
-            sys.stderr.write("\n")
-            return 255
-        except NoRegionError as e:
-            msg = ('%s You can also configure your region by running '
-                   '"aws configure".' % e)
-            self._show_error(msg)
-            return 255
-        except NoCredentialsError as e:
-            msg = ('%s. You can configure credentials by running '
-                   '"aws configure".' % e)
-            self._show_error(msg)
-            return 255
-        except KeyboardInterrupt:
-            # Shell standard for signals that terminate
-            # the process is to return 128 + signum, in this case
-            # SIGINT=2, so we'll have an RC of 130.
-            sys.stdout.write("\n")
-            return 128 + signal.SIGINT
-        except Exception as e:
+        except BaseException as e:
+            # when --version action executed default argparser prints out
+            # version string and calls sys.exit(0)
+            if isinstance(e, SystemExit) and e.code == 0:
+                return e.code
             LOG.debug("Exception caught in main()", exc_info=True)
-            LOG.debug("Exiting with rc 255")
-            write_exception(e, outfile=get_stderr_text_writer())
-            return 255
+            return self._error_handler.handle_exception(
+                e,
+                stdout=get_stdout_text_writer(),
+                stderr=get_stderr_text_writer()
+            )
 
     def _emit_session_event(self, parsed_args):
         # This event is guaranteed to run after the session has been
@@ -249,27 +485,30 @@ class CLIDriver(object):
 
     def _handle_top_level_args(self, args):
         emit_top_level_args_parsed_event(self.session, args)
-        if args.profile:
+        if getattr(args, 'profile', False):
             self.session.set_config_variable('profile', args.profile)
-        if args.region:
+        if getattr(args, 'region', False):
             self.session.set_config_variable('region', args.region)
-        if args.debug:
-            # TODO:
-            # Unfortunately, by setting debug mode here, we miss out
-            # on all of the debug events prior to this such as the
-            # loading of plugins, etc.
-            self.session.set_stream_logger('botocore', logging.DEBUG,
-                                           format_string=LOG_FORMAT)
-            self.session.set_stream_logger('awscli', logging.DEBUG,
-                                           format_string=LOG_FORMAT)
-            self.session.set_stream_logger('s3transfer', logging.DEBUG,
-                                           format_string=LOG_FORMAT)
+        self._set_logging(getattr(args, 'debug', False))
+
+    def _set_logging(self, debug):
+        loggers_list = ['botocore', 'awscli', 's3transfer', 'urllib3']
+        if debug:
+            for logger_name in loggers_list:
+                set_stream_logger(logger_name, logging.DEBUG,
+                                  format_string=LOG_FORMAT)
+            enable_crt_logging()
             LOG.debug("CLI version: %s", self.session.user_agent())
             LOG.debug("Arguments entered to CLI: %s", sys.argv[1:])
-
         else:
-            self.session.set_stream_logger(logger_name='awscli',
-                                           log_level=logging.ERROR)
+            # In case user set --debug before entering prompt mode and removed
+            # it during editing inside the prompter we need to remove all the
+            # debug handlers
+            for logger_name in loggers_list:
+                remove_stream_logger(logger_name)
+            disable_crt_logging()
+            set_stream_logger(logger_name='awscli',
+                              log_level=logging.ERROR)
 
 
 class ServiceCommand(CLICommand):
@@ -323,6 +562,16 @@ class ServiceCommand(CLICommand):
     def lineage(self, value):
         self._lineage = value
 
+    @property
+    def subcommand_table(self):
+        return self._get_command_table()
+
+    @property
+    def arg_table(self):
+        # No service level arguments so we just return an empty
+        # dictionary.
+        return {}
+
     def _get_command_table(self):
         if self._command_table is None:
             self._command_table = self._create_command_table()
@@ -330,17 +579,15 @@ class ServiceCommand(CLICommand):
 
     def _get_service_model(self):
         if self._service_model is None:
-            api_version = self.session.get_config_variable('api_versions').get(
-                self._service_name, None)
             self._service_model = self.session.get_service_model(
-                self._service_name, api_version=api_version)
+                self._service_name)
         return self._service_model
 
     def __call__(self, args, parsed_globals):
         # Once we know we're trying to call a service for this operation
         # we can go ahead and create the parser for it.  We
         # can also grab the Service object from botocore.
-        service_parser = self._create_parser()
+        service_parser = self.create_parser()
         parsed_args, remaining = service_parser.parse_known_args(args)
         command_table = self._get_command_table()
         return command_table[parsed_args.operation](remaining, parsed_globals)
@@ -379,7 +626,7 @@ class ServiceCommand(CLICommand):
                                   event_class='.'.join(self.lineage_names),
                                   name=self._name)
 
-    def _create_parser(self):
+    def create_parser(self):
         command_table = self._get_command_table()
         # Also add a 'help' command.
         command_table['help'] = self.create_help_command()
@@ -457,6 +704,12 @@ class ServiceOperation(object):
         # Represents the lineage of a command in terms of command ``name``
         return [cmd.name for cmd in self.lineage]
 
+    @property
+    def subcommand_table(self):
+        # There's no subcommands for an operation so we return an
+        # empty dictionary.
+        return {}
+
     @property
     def arg_table(self):
         if self._arg_table is None:
@@ -487,7 +740,6 @@ class ServiceOperation(object):
                    parsed_globals=parsed_globals)
         call_parameters = self._build_call_parameters(
             parsed_args, self.arg_table)
-
         event = 'calling-command.%s.%s' % (self._parent_name,
                                            self._name)
         override = self._emit_first_non_none_response(
@@ -500,6 +752,8 @@ class ServiceOperation(object):
         # of exception that will be raised if detected or it can represent
         # the desired return code. Note that a return code of 0 represents
         # a success.
+        if hasattr(self._session, 'user_agent_extra'):
+            self._add_customization_to_user_agent()
         if override is not None:
             if isinstance(override, Exception):
                 # If the override value provided back is an exception then
@@ -598,6 +852,14 @@ class ServiceOperation(object):
         parser = ArgTableArgParser(arg_table)
         return parser
 
+    def _add_customization_to_user_agent(self):
+        if ' command/' in self._session.user_agent_extra:
+            self._session.user_agent_extra += '.%s' % self.lineage_names[-1]
+        else:
+            self._session.user_agent_extra += ' command/%s' % '.'.join(
+                self.lineage_names
+            )
+
 
 class CLIOperationCaller(object):
 
@@ -605,23 +867,24 @@ class CLIOperationCaller(object):
 
     def __init__(self, session):
         self._session = session
+        self._output_stream_factory = OutputStreamFactory(session)
 
     def invoke(self, service_name, operation_name, parameters, parsed_globals):
         """Invoke an operation and format the response.
 
         :type service_name: str
-        :param service_name: The name of the service.  Note this is the service name,
-            not the endpoint prefix (e.g. ``ses`` not ``email``).
+        :param service_name: The name of the service.  Note this is the
+            service name, not the endpoint prefix (e.g. ``ses`` not ``email``).
 
         :type operation_name: str
         :param operation_name: The operation name of the service.  The casing
-            of the operation name should match the exact casing used by the service,
-            e.g. ``DescribeInstances``, not ``describe-instances`` or
+            of the operation name should match the exact casing used by the
+            service, e.g. ``DescribeInstances``, not ``describe-instances`` or
             ``describe_instances``.
 
         :type parameters: dict
-        :param parameters: The parameters for the operation call.  Again, these values
-            have the same casing used by the service.
+        :param parameters: The parameters for the operation call.  Again,
+            these values have the same casing used by the service.
 
         :type parsed_globals: Namespace
         :param parsed_globals: The parsed globals from the command line.
@@ -655,5 +918,7 @@ class CLIOperationCaller(object):
         output = parsed_globals.output
         if output is None:
             output = self._session.get_config_variable('output')
+
         formatter = get_formatter(output, parsed_globals)
-        formatter(command_name, response)
+        with self._output_stream_factory.get_output_stream() as stream:
+            formatter(command_name, response, stream)
